header {
/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2007,2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
 
package zero.core.config.antlr;
}

/* #################################################################### */
/* #                              Lexer                                 */
/* #################################################################### */
{
import zero.core.config.ConfigLoader.Handler;
}
class Config2Lexer extends Lexer;

options {
	charVocabulary = '\0'..'\ufffe';  // uffff is EOF
	defaultErrorHandler=false;
	k=2;
	testLiterals=false;
}

tokens {
	DOUBLE;
}

{
	private Handler handler;
	
	public void setHandler(Handler handler) {
		this.handler = handler;
	}
	public Handler getHandler() {
		return handler;
	}
}

COMMA options { paraphrase = ","; } : ',';
LBRACE options { paraphrase = "{"; } : '{';
RBRACE options { paraphrase = "}"; } : '}';
LBRACKET options { paraphrase = "["; } : '[';
RBRACKET options { paraphrase = "]"; } : ']';
COLON options { paraphrase = ":"; } : ':';
ASSIGN options { paraphrase = "="; } : '=';
APPEND options { paraphrase = "+="; } : "+=";
INCLUDE options { paraphrase = "@include"; } : "@include";

GCPath 
	: '/' GCPathElements
	;
	
protected GCPathElements
	// Defined as all characters except that end a Reference or may follow a GCPath
	: (("${") => r:Reference! {$append(handler.getReferenceValue(r)); }
	| EscapeSequence | ~('\u0000'..'\u001f' | ' ' | '\\' | '+' | '=' | '$' | '}') )+
	;



Reference 
	: "${"! (Variable |  GCPath) RBRACE!
	;

Number
	: ('-')? (Digit)+ ( '.' (Digit)+ {$setType(DOUBLE);})? (Exponent {$setType(DOUBLE);})?;

protected Exponent: ('e'|'E') ('-')? (Digit)+;

protected Digit
	: '0'..'9'
	;
		
String
	: '"'! ( r:Reference! {$append(handler.getReferenceValue(r)); } 
			| EscapeSequence
			| ~('\u0000'..'\u001f' | '$' | '\\' | '\"' )
		)* '"'!
	;
	
protected EscapeSequence
	:	"$$" {$setText('$');} 
	|	'\\' (UnicodeEscape 
			|'b' {$setText('\b');}
			|'t' {$setText('\t');}
			|'n' {$setText('\n');}
			|'f' {$setText('\f');}
			|'r' {$setText('\r');}
			|'\"' {$setText('\"');}
			|'\'' {$setText('\'');}
			|'\\' {$setText('\\');}
		)
	;

protected UnicodeEscape
	: 'u' HexDigit HexDigit HexDigit HexDigit { 
   		String s = $getText;
		char codeValue = (char) Integer.parseInt(s.substring(1), 16);
		$setText(codeValue);
	}
	;

protected HexDigit
	: '0'..'9' | 'A'..'F' | 'a'..'f'
	;
 
// Allow true, false and null not to be treated as variables
Variable options { testLiterals=true; }
	: ('a'..'z' | 'A'..'Z') ('a'..'z' | 'A'..'Z' | '0'..'9' | '_')* 
	;
	
WS : ( ' '
    | '\t' { tab(); }
    | '\f'

    // handle newlines
    | ( "\r\n"  // DOS/Windows
      | '\r'    // Macintosh
      | '\n'    // Unix
      )
      // increment the line count in the scanner
      { newline(); }
    )
    { $setType(Token.SKIP); }
  ;

COMMENT
  : "#" (~('\n'|'\r'))*
    { $setType(Token.SKIP); }
  ;


{
import java.util.List;
import java.util.Map;

import zero.core.config.ConfigLoader.Handler;
}
class Config2Parser extends Parser; //("antlr.debug.ParseTreeDebugParser");
options {
	defaultErrorHandler=false;
}

{
	private Handler handler;
	
	public Config2Parser(Config2Lexer lexer) {
		this((TokenStream) lexer);
		this.handler = lexer.getHandler();
	}
			
}

config
	: (gcop
	| varset
	| include)* EOF!
	;

gcop
	{ Object v;}
	: (r:Reference | g:GCPath) (assign:ASSIGN! | APPEND!) v=value
		{
			Token token;
			String key;
			if (r != null) {
				token = r;
				key = handler.getReferenceValue(r);
			} else {
				token = g;
				key = g.getText();
			} 
			if (assign != null) handler.gcSet(token, key, v); else handler.gcAppend(token, key, v); 
		}
	;
	
varset
	{ Object v; }
	: ( var:Variable ASSIGN! v=value ) { handler.variableSet(var, v); }
	;
	
include
	{ Object o = null; }
	: INCLUDE! s:String (o=object)? { handler.includeFile(s, o); }
	;
	
value returns [Object val = null]
	: s:String { val = s.getText(); }
	| n:Number {val = handler.number(n); }
	| d:DOUBLE {val = handler.number(d); }
	| "true" {val = Boolean.TRUE; }
	| "false" {val = Boolean.FALSE; }
	| "null" { val = null; }
	| val=object
	| val=array
	| val=reference
	;

reference returns [Object val = null]
	: r:Reference 
		{
			val = handler.getReference(r);
		} 
	;

/* &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/* Start: json.g                                                        */
/* &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& */
/*
string returns [String string = null]
	{ StringBuilder sb = new StringBuilder(); Object r;	}
	: "\""! 
		(("${") => r=ref { sb.append(r); } 
		| l:Literals {sb.append(l.getText()); }
		)*
		"\""!
	{ string = sb.toString(); }
	;
*/	

object returns [Map<Object, Object> map = handler.newMap()]
	{ Object[] p, p1; }
	: LBRACE! (p=pair { handler.mapPut(map, p); }
		 (COMMA! p1=pair {handler.mapPut(map, p1);})*)? RBRACE!

	;
	
pair returns [Object[] pair=null]
	{ Object v; }
	: s:String COLON v=value { pair = handler.newPair(s.getText(), v); }
	;
	
array returns [List<Object> list = handler.newList()]
	{ Object v; Object v1; }
	: LBRACKET! (v=value { handler.listAdd(list, v); } 
		(COMMA! v1=value { handler.listAdd(list, v1); } 
			)*)? RBRACKET!
	;
