header {
/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * Project  Zero
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
 
package zero.core.events.antlr;
}


/* #################################################################### */
/* #                              Lexer                                 */
/* #################################################################### */
{
import antlr.TokenStreamSelector;
}
class ConditionsLexer extends Lexer;

options {
	k=2;  // k=3 to distinguish GCKeyRef ${/ from VariableRef ${a
	charVocabulary = '\0'..'\ufffe';  // uffff is EOF
	defaultErrorHandler=false;
	importVocab=ValueLexer;
}

{
TokenStreamSelector _selector = null;

public void setSelector(TokenStreamSelector selector) {
	_selector = selector;
}
}


And
	options { paraphrase = "'&&'"; }
	: "&&"
	;
	
Or
	options { paraphrase = "'||'"; }
	: "||"
	;
	
Not
	options { paraphrase = "'!'"; }
	: "!"
	;

SelectorMatch
	options { paraphrase = "'=='"; }
	: "==" { _selector.push("value"); }
	;

SelectorNotMatch
	options { paraphrase = "'!='"; }
	: "!=" { _selector.push("value"); }
	;

RegexMatch
	options { paraphrase = "'=~'"; }
	: "=~" { _selector.push("value"); }
	;

RegexNotMatch
	options { paraphrase = "'!~'"; }
	: "!~" { _selector.push("value"); }
	;
	
LParen
	options { paraphrase = "'('"; }
	: '('
	;

RParen
	options { paraphrase = "')'"; }
	: ')'
	;

GCKey 
	: ('/' GCPathElement (('/') GCPathElement)*) ;
	
protected GCPathElement
	: (~('/' | '}' | '+' | '=' | ' ' | ')'))+ ;
	
WS 
	: ( ' ' ) { $setType(Token.SKIP); }
	;
	
	
/* #################################################################### */
/* #                             Parser                                 */
/* #################################################################### */
{
import antlr.TokenStreamSelector;

	import java.util.List;
	import java.util.logging.Level;
	import java.util.logging.Logger;

	import zero.core.events.AndCondition;
	import zero.core.events.Condition;
	import zero.core.events.ExistsCondition;
	import zero.core.events.NotCondition;
	import zero.core.events.OrCondition;
	import zero.core.events.RegexCondition;
	import zero.core.events.SelectorCondition;

	import zero.core.logging.LogHelper;

}
class ConditionsParser extends Parser; //("zero.antlr.Parser");
	
options {
	defaultErrorHandler=false;
}

{
private static final String CLASS_NAME = ConditionsParser.class.getName();
private static final Logger _logger = Logger.getLogger(CLASS_NAME, "zero.kernel.messages");
		
TokenStreamSelector _selector = null;

public void setSelector(TokenStreamSelector selector) {
	_selector = selector;
}
}


condition returns [Condition c=null]
	: g:GCKey ((sm:SelectorMatch | snm:SelectorNotMatch | rm:RegexMatch | rnm:RegexNotMatch) val:Value)?
	{ String gcPath = g.getText(); String v = (val != null) ? val.getText().trim() : null;
	  if (sm != null) {c = new SelectorCondition(gcPath, v);
	  } else if (snm != null) { c = new NotCondition(new SelectorCondition(gcPath, v));
	  } else if (rm != null) { c = new RegexCondition(gcPath, v);
	  } else if (rnm != null) {c = new NotCondition(new RegexCondition(gcPath, v));
	  } else {c = new ExistsCondition(gcPath);
	  }
	}
	
	;
	
condition_predicate returns [Condition c=null]
	{ Condition nextCondition; }
	: c=condition_group 
	  ((a:And | Or) 
	    nextCondition=condition_group 
	    { c = (a != null) ? new AndCondition(c, nextCondition) : new OrCondition(c, nextCondition);  
     	  a = null;
	    }
	  )*
	;
	
condition_group returns [Condition c=null]
	: (n:Not)? LParen! (c=condition | c=condition_predicate) RParen!
	{
		if (n != null) { c = new NotCondition(c); }
	}
	;

// Need to guarantee that we have parsed everything
rule returns [Condition c=null]
	: (c=condition | c=condition_predicate) EOF!
	;
